import { BehaviorSubject, Observable } from 'rxjs';
import { GetLoggerAsyncOptions } from '../options';
export declare namespace loggerTypes {
    interface Counters<T> {
        pageReads: T;
        lastPageReads: T;
        customReads: T;
        lastCustomReads: T;
        sessionReads: T;
        lastSessionReads: T;
    }
    export type CounterSubjects = Counters<BehaviorSubject<number>>;
    export type CounterStreams = Counters<Observable<number>>;
    export type CounterEmitOptions = Counters<boolean>;
    export type ResetFn = () => void;
    export type IncrementFn = (newReads: number) => void;
    export interface ReadsLogger {
        pageReads: number;
        lastPageReads: number;
        resetPageReads: ResetFn;
        customReads: number;
        lastCustomReads: number;
        resetCustomReads: ResetFn;
        sessionReads: number;
        lastSessionReads: number;
        incrementAll: IncrementFn;
        counters$: CounterStreams;
    }
    export type GetLoggerAsyncFn = (retryCount: number) => Promise<ReadsLogger>;
    export {};
}
export declare namespace loggerConstants {
    const EMIT_OPTIONS_FROM_STORAGE: loggerTypes.CounterEmitOptions;
    const EMIT_OPTIONS_ALL: loggerTypes.CounterEmitOptions;
    const DEFAULT_GET_LOGGER_ASYNC_OPTIONS: GetLoggerAsyncOptions;
}
export declare namespace loggerStreamHelpers {
    type CreateSubjectsFn = () => loggerTypes.CounterSubjects;
    type GetStreamsFn = (subjects: loggerTypes.CounterSubjects) => loggerTypes.CounterStreams;
    export const createSubjects: CreateSubjectsFn;
    export const getStreams: GetStreamsFn;
    export {};
}
